/* routines for testing an amphibian sender - sends "hello"
   to an address given on the command line, and tries to
   read hello back */
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/time.h>
#include <string.h>
#include <errno.h>
#include "amphibian_app.h"
#include "fishperf.h"

#define MAX_XFER (128 * 1024)

int main(int argc __attribute__((unused)), 
         const char *argv[] __attribute__((unused))) {
  int sk;
  struct sockaddr_fish sf;
  int zok;
  char *sendbuf = malloc(MAX_XFER);
  char *recvbuf = malloc(MAX_XFER);
  int readbytes, filled;
  int size;
  double deltat;
  struct timeval tv_start,tv_end;
  int max_xfer = MAX_XFER;
  int i;
  sf.sfish_family = AF_FISH;
  sf.sfish_address = atoi(argv[1]);
  if(argc >= 3) {
    max_xfer = atoi(argv[2]);
    assert(max_xfer <= MAX_XFER);
  }
  if(argc >=4 ) {
    sf.sfish_port = FISH_SERVICE_ECHO;
  } else {
    sf.sfish_port = FISH_SERVICE_DISCARD;
  }
  for(i=0; i< MAX_XFER; i++) {
    sendbuf[i] = char_from_index(i);
  }
  for(size = 4096; size <= max_xfer; size *= 2) {
    sk = fishsockets_socket(AF_FISH, SOCK_STREAM, 0);
    if(sk < 0) fprintf(stderr, "client: socket returned %d: %s\n", sk, strerror(errno));

    zok = fishsockets_connect(sk, (struct sockaddr *)&sf, sizeof(struct sockaddr_fish));
    if(zok != 0) {
      fprintf(stderr, "client: connect returned %d: %s\n", zok, strerror(errno));
      fishsockets_close(sk); 
      exit(EXIT_FAILURE);
    }

    gettimeofday(&tv_start, NULL);

    zok = fishsockets_write(sk, sendbuf, size); 
    if(zok < size) 
      fprintf(stderr, "client: write returned %d: %s\n", zok, strerror(errno));

    if(sf.sfish_port == FISH_SERVICE_ECHO) {
      readbytes = 0;
      for(filled = 0; filled < size; filled += readbytes) {
        readbytes = fishsockets_read(sk, recvbuf, size - filled); 
        if(readbytes < 0) {
          fprintf(stderr, "client: read gave %d: %s\n", zok, strerror(errno));
          exit(EXIT_FAILURE);
        }
        if(readbytes == 0) {
          break;
        }
      }
    } else {
      filled = zok;
    }

    fishsockets_close(sk); 
    gettimeofday(&tv_end, NULL);

    deltat = (double)(tv_end.tv_sec - tv_start.tv_sec) + 
      (double)(tv_end.tv_usec - tv_start.tv_usec)/1000000.0;
    printf("client: sent %d bytes in %5.3f seconds, at %5.3f megabits per second\n", 
           filled, deltat, (float)filled/deltat/1000000.0*8.0);
  }
  exit(EXIT_SUCCESS);
}
  
